package factory.clients;

import java.rmi.RemoteException;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;

import factory.manage_orders.ManageOrders;
import factory.manage_orders.ManageOrdersHome;
import factory.manage_orders.OverdueOrderView;
import factory.manage_orders.OpenOrderView;

public class ManageSampleOrders
{
  public static void main(String[] args)
  {
    try
    {
      ContextPropertiesFactory factory = new ContextPropertiesFactory();
      Properties prop = factory.getInitialContextProperties();
      InitialContext initial = new InitialContext( prop );

      Object homeObject = initial.lookup("ManageOrders");

      ManageOrdersHome home = (ManageOrdersHome)
        PortableRemoteObject.narrow(homeObject, ManageOrdersHome.class);

      ManageOrders manageOrders = home.create();

      // List overdue orders
      OverdueOrderView[] overdueOrders = manageOrders.getOverdueOrders();
      for(int iter = 0; iter < overdueOrders.length; iter++)
      {
        OverdueOrderView overdueOrder = overdueOrders[iter];
        System.out.println("Product " + overdueOrder.product + 
                           " is due on " + overdueOrder.dateDue + 
                           ".  It's status is " + overdueOrder.status 
                           + ".");
      }

      // Cancel first overdue order
      if(overdueOrders.length > 0)
      {
        OverdueOrderView overdueOrder = overdueOrders[0];
        System.out.println("About to cancel an order...");
        try
        {
          manageOrders.cancelOrder(overdueOrder.salesDivision, 
                                   overdueOrder.orderNumber );
          System.out.println("Canceled order for " + 
                             overdueOrder.product.trim() + ".");
        }
        catch(factory.manage_orders.NoSuchOrderException nsoe)
        {
          System.out.println("Failed to find order.");
        }
        catch(factory.order.OrderNotCancelableException once)
        {
          System.out.println("Cannot cancel an order in production.");
        }
      }

    }
    catch(RemoteException re)
    {
      re.printStackTrace();
    }
    catch(NamingException ne)
    {
      ne.printStackTrace();
    }
    catch(CreateException ce)
    {
      ce.printStackTrace();
    }
  }
}
