package factory.manufacture;

import javax.ejb.*;
import javax.naming.*;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import javax.sql.DataSource;
import java.util.Arrays;
import java.util.List;
import java.util.Collections;
import java.util.Iterator;
import java.util.Date;
import factory.product.Product;
import factory.product.ProductHome;
import factory.order.Order;
import factory.order.OrderHome;
import factory.order.OrderPK;
import factory.order.StatusStrings;
import factory.manage_orders.OpenOrderView;
import factory.manage_orders.ManageOrders;
import factory.manage_orders.ManageOrdersHome;
import factory.manage_orders.NoSuchOrderException;
import factory.product.RoutingInstruction;

public class ManufactureEJB implements SessionBean
{
  // properties
  public String manufactureCellID;
  public List routingInstructions;
  public int currentPosition;
  public int lastPosition;

  public boolean orderSelected;
  public int selectedSalesDivision;
  public int selectedOrderNumber;

// Framework & lifecycle methods

  public void ejbCreate(String manufactureCellID)
  {
    this.manufactureCellID = manufactureCellID;
    orderSelected = false;
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  public void setSessionContext(SessionContext ctx)
  {
  }
// Business methods

  public OpenOrderView[] getOpenOrders()
  {
    try
    {
      ManageOrdersHome homeManageOrders = getManageOrdersHome();
      ManageOrders manageOrders = homeManageOrders.create();
      return manageOrders.getSchedulableOrders();
    }
    catch(NamingException ne)
    {
      throw new EJBException(ne);
    }
    catch(RemoteException re)
    {
      throw new EJBException(re);
    }
    catch(CreateException ce)
    {
      throw new EJBException(ce);
    }
  }

  public void selectForManufacture(int salesDivision, int order_number)
              throws NoSuchOrderException, BadStatusException
  {
    try
    {
      OrderHome homeOrder = getOrderHome();
      OrderPK orderPK = new OrderPK(salesDivision, order_number);
      Order order = homeOrder.findByPrimaryKey(orderPK);
      String orderStatus = order.getStatus();
      if(!orderStatus.equals(StatusStrings.OPEN))
         throw new BadStatusException(orderStatus);
      order.beginManufacture();
      Product product = order.getProductOrdered();
      RoutingInstruction[] productRouting =
                           product.getRoutingInstructions();
      routingInstructions = Arrays.asList(productRouting);
      Collections.sort(routingInstructions);
      currentPosition = 0;
      lastPosition = routingInstructions.size() - 1;
      selectedSalesDivision = salesDivision;
      selectedOrderNumber = order_number;
      orderSelected = true;
    }
    catch(NamingException ne)
    {
      ne.printStackTrace();
      throw new EJBException(ne);
    }
    catch(RemoteException re)
    {
      re.printStackTrace();
      throw new EJBException(re);
    }
    catch(FinderException fe)
    {
      fe.printStackTrace();
      if(fe instanceof ObjectNotFoundException)
         throw new NoSuchOrderException();
      else
         throw new EJBException(fe);
    }
  }

  public boolean hasNextRouting() throws NoSelectionException
  {
    if(!orderSelected)
       throw new NoSelectionException();
    return (currentPosition <= lastPosition);
  }

  public String getNextRouting() throws NoSelectionException
  {
    if(!orderSelected)
       throw new NoSelectionException();

    RoutingInstruction ri = (RoutingInstruction)
                            routingInstructions.get( currentPosition++ );
    return ri.instruction;
  }

  public void ship(String carrier, int loading_dock)
              throws NoSelectionException
  {
    // Sales division, order number, carrier,
    // loading dock, date completed, manufactured by

    if(!orderSelected)
       throw new NoSelectionException();

    Connection con = null;

    try
    {
      con = getConnection();
      PreparedStatement statement = con.prepareStatement(
                                        getShipmentSQLString());
      statement.setInt(1, selectedSalesDivision);
      statement.setInt(2, selectedOrderNumber);
      statement.setString(3, carrier);
      statement.setInt(4, loading_dock);
      statement.setDate(5, new java.sql.Date((new Date()).getTime()));
      statement.setString(6, manufactureCellID);
      statement.executeUpdate();
      statement.close();
      con.close();
      orderSelected = false;

      OrderHome homeOrder = getOrderHome();
      OrderPK orderPK = new OrderPK(
                            selectedSalesDivision, selectedOrderNumber);
      Order order = homeOrder.findByPrimaryKey(orderPK);
      order.completeManufacture();
    }
    catch(NamingException ne)
    {
      ne.printStackTrace();
      throw new EJBException(ne);
    }
    catch(SQLException sqle)
    {
      try {
        if(con != null)
          con.close();} catch (Exception e){}
      sqle.printStackTrace();
      throw new EJBException(sqle);
    }
    catch(RemoteException re)
    {
      throw new EJBException(re);
    }
    catch(FinderException fe)
    {
      throw new EJBException(fe);
    }
  }

// Implementation helper
  private OrderHome getOrderHome() throws NamingException
  {
    InitialContext initial = new InitialContext();
    OrderHome home = (OrderHome)javax.rmi.PortableRemoteObject.narrow(
                      initial.lookup("java:comp/env/ejb/Order"), 
                                     OrderHome.class );
    return home;
  }

  private ManageOrdersHome getManageOrdersHome() throws NamingException
  {
    InitialContext initial = new InitialContext();
    ManageOrdersHome home = (ManageOrdersHome)  
                             javax.rmi.PortableRemoteObject.narrow(
                             initial.lookup(
                                    "java:comp/env/ejb/ManageOrders"),  
                                    ManageOrdersHome.class );
    return home;
  }

  private Connection getConnection() throws SQLException, NamingException
  {
    Context initial = new InitialContext();
    DataSource dataSource =
              (DataSource)initial.lookup("java:comp/env/jdbc/shipDB");
    return dataSource.getConnection();
  }

  private String getShipmentSQLString() throws NamingException
  {
    InitialContext initial = new InitialContext();
    String sql = (String) initial.lookup("java:comp/env/shipmentSQL");
    return sql;
  }
}
