package book.order;

import javax.ejb.*;
import javax.naming.*;
import java.rmi.RemoteException;
import book.product.Product;
import book.product.ProductHome;
import java.util.Date;

public class OrderEJB implements EntityBean
{
  // properties
  private static final String OPEN_STATUS = "o";
  private static final String DEFAULT_STATUS = OPEN_STATUS;
  private static final String CANCELED_STATUS = "c";
  private static final String IN_PROCESS_STATUS = "m";
  private static final String COMPLETED_STATUS = "f";

  public int salesDivision;
  public int orderNumber;
  public Product productOrdered;
  public String status;
  public Date dateDue;

  // business methods

  public int getSalesDivision()
  {
    return salesDivision;
  }

  public int getOrderNumber()
  {
    return orderNumber;
  }

  public Product getProductOrdered()
  {
    return productOrdered;
  }

  public String getStatus()
  {
    if (status.equals( OPEN_STATUS ))
      return StatusStrings.OPEN;
    else if (status.equals( CANCELED_STATUS ))
      return StatusStrings.CANCELED;
    else if (status.equals( IN_PROCESS_STATUS ))
      return StatusStrings.IN_PROCESS;
    else if (status.equals( COMPLETED_STATUS ))
      return StatusStrings.COMPLETED;
    throw new EJBException( "Unknown status" );
  }

  public void cancelOrder()
    throws OrderNotCancelableException
  {
    if (
      status.equals( IN_PROCESS_STATUS )
        ||
      status.equals( COMPLETED_STATUS )
        )
      throw new OrderNotCancelableException();
    status = CANCELED_STATUS;
  }

  public void beginManufacture()
  {
    status = IN_PROCESS_STATUS;
  }

  public void completeManufacture()
  {
    status = COMPLETED_STATUS;
  }

  public Date getDateDue()
  {
    return dateDue;
  }

  // implementation helpers

  private ProductHome getProductHome()
  {
    try
    {
      InitialContext initial = new InitialContext();
      ProductHome home = (ProductHome) javax.rmi.PortableRemoteObject.narrow(
        initial.lookup( "java:comp/env/ejb/Product" ), ProductHome.class );
      return home;
    }
    catch (NamingException ne)
    {
      ne.printStackTrace();
      throw new EJBException( ne );
    }
  }

  // lifecycle and framework methods

  public OrderPK ejbCreate( int salesDivision, int orderNumber,
    Product productOrdered, Date dateDue )
    throws CreateException
  {
    this.salesDivision = salesDivision;
    this.orderNumber = orderNumber;
    this.productOrdered = productOrdered;
    this.dateDue = dateDue;
    status = DEFAULT_STATUS;

    return null; // for container-managed persistence
  }

  public void ejbPostCreate( int salesDivision, int orderNumber,
    Product productOrdered, Date dateDue )
  {
  }

  public OrderPK ejbCreate( int salesDivision, int orderNumber,
    String product, Date dateDue )
    throws CreateException
  {
    this.salesDivision = salesDivision;
    this.orderNumber = orderNumber;
    this.dateDue = dateDue;
    status = DEFAULT_STATUS;

    try
    {
      ProductHome productHome = getProductHome();
      this.productOrdered = productHome.findByPrimaryKey( product );
    }
    catch (RemoteException re)
    {
      re.printStackTrace();
      throw new EJBException( re );
    }
    catch (FinderException fe)
    {
      fe.printStackTrace();
      throw new CreateException( "Product does not exist" );
    }

    return null; // for container-managed persistence
  }

  public void ejbPostCreate( int salesDivision, int orderNumber,
    String product, Date dateDue )
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbLoad()
  {
    status = status.trim();
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbStore()
  {
  }

  public void setEntityContext(EntityContext ctx)
  {
  }

  public void unsetEntityContext()
  {
  }
} 